@@ -45,7 +45,7 @@ struct target {
 
 static struct target *targets=NULL;
 
-static tag tag_table[TABLESIZE];
+static tag *tag_table = NULL;
 
 #define gettag2(s1,s2) (((s1) + (s2)) & 0xFFFF)
 #define gettag(sum) gettag2((sum)&0xFFFF,(sum)>>16)
@@ -60,8 +60,12 @@ static void build_hash_table(struct sum_struct *s)
 {
   int i;
 
+  if (!tag_table)
+    tag_table = (tag *)malloc(sizeof(tag)*TABLESIZE);
+
   targets = (struct target *)malloc(sizeof(targets[0])*s->count);
-  if (!targets) out_of_memory("build_hash_table");
+  if (!tag_table || !targets) 
+    out_of_memory("build_hash_table");
 
   for (i=0;i<s->count;i++) {
     targets[i].i = i;
