@@ -203,19 +203,14 @@ static void set_perms(char *fname,struct file_struct *file,struct stat *st,
     st = &st2;
   }
 
-#ifdef HAVE_UTIME
   if (preserve_times && st->st_mtime != file->modtime) {
-    struct utimbuf tbuf;  
     updated = 1;
-    tbuf.actime = time(NULL);
-    tbuf.modtime = file->modtime;
-    if (utime(fname,&tbuf) != 0) {
+    if (set_modtime(fname,file->modtime) != 0) {
       fprintf(stderr,"failed to set times on %s : %s\n",
 	      fname,strerror(errno));
       return;
     }
   }
-#endif
 
 #ifdef HAVE_CHMOD
   if (preserve_perms && st->st_mode != file->mode) {
@@ -472,6 +467,15 @@ static void delete_files(struct file_list *flist)
   }
 }
 
+static char *cleanup_fname = NULL;
+
+void sig_int(void)
+{
+  if (cleanup_fname)
+    unlink(cleanup_fname);
+  exit(1);
+}
+
 
 int recv_files(int f_in,struct file_list *flist,char *local_name)
 {  
@@ -509,25 +513,21 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
 	fprintf(stderr,"recv_files(%s)\n",fname);
 
       /* open the file */  
-      if ((fd1 = open(fname,O_RDONLY)) == -1 &&
-	  (fd1 = open(fname,O_RDONLY|O_CREAT,flist->files[i].mode)) == -1) {
-	fprintf(stderr,"recv_files failed to open %s\n",fname);
-	return -1;
-      }
+      fd1 = open(fname,O_RDONLY);
 
-      if (fstat(fd1,&st) != 0) {
+      if (fd1 != -1 && fstat(fd1,&st) != 0) {
 	fprintf(stderr,"fstat %s : %s\n",fname,strerror(errno));
 	close(fd1);
 	return -1;
       }
 
-      if (!S_ISREG(st.st_mode)) {
+      if (fd1 != -1 && !S_ISREG(st.st_mode)) {
 	fprintf(stderr,"%s : not a regular file\n",fname);
 	close(fd1);
 	return -1;
       }
 
-      if (st.st_size > 0) {
+      if (fd1 != -1 && st.st_size > 0) {
 	buf = map_file(fd1,st.st_size);
 	if (!buf) {
 	  fprintf(stderr,"map_file failed\n");
@@ -551,6 +551,8 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
 	fprintf(stderr,"open %s : %s\n",fnametmp,strerror(errno));
 	return -1;
       }
+      
+      cleanup_fname = fnametmp;
 
       if (!am_server && verbose)
 	printf("%s\n",fname);
@@ -558,7 +560,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
       /* recv file data */
       receive_data(f_in,buf,fd2,fname);
 
-      close(fd1);
+      if (fd1 != -1) close(fd1);
       close(fd2);
 
       if (verbose > 2)
@@ -567,7 +569,7 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
       if (make_backups) {
 	char fnamebak[MAXPATHLEN];
 	sprintf(fnamebak,"%s%s",fname,backup_suffix);
-	if (rename(fname,fnamebak) != 0) {
+	if (rename(fname,fnamebak) != 0 && errno != ENOENT) {
 	  fprintf(stderr,"rename %s %s : %s\n",fname,fnamebak,strerror(errno));
 	  exit(1);
 	}
@@ -579,6 +581,8 @@ int recv_files(int f_in,struct file_list *flist,char *local_name)
 		fnametmp,fname,strerror(errno));
       }
 
+      cleanup_fname = NULL;
+
       unmap_file(buf,st.st_size);
 
       set_perms(fname,&flist->files[i],NULL,0);
