@@ -88,7 +88,9 @@
 #include <errno.h>
 
 #include <sys/mman.h>
+#ifdef HAVE_UTIME_H
 #include <utime.h>
+#endif
 
 #ifdef HAVE_FNMATCH
 #include <fnmatch.h>
@@ -224,3 +226,5 @@ extern int errno;
 #if !SUPPORT_LINKS
 #define lstat stat
 #endif
+
+#define SIGNAL_CAST (RETSIGTYPE (*)())
